#!/usr/bin/env bash
#set -x

# Copyright (c) 2009 & onwards. MapR Tech, Inc., All rights reserved

############################################################################
#
#
# Configure script for Hive
#
##
# This script is normally run by the core configure.sh to setup Hive during
# install. If it is run standalone, need to correctly initialize the
# variables that it normally inherits from the master configure.sh
#
############################################################################

MAPR_HOME="${BASEMAPR:-/opt/mapr}"
MAPR_ENABLE_LOGS="${MAPR_ENABLE_LOGS:-false}"

configure_logging(){
{ set +x; } 2> /dev/null
if [ "$MAPR_ENABLE_LOGS" == "true" ]; then
  set -x;
fi
}

#
# Executes bash script and log error if file does not exit. Hides the output.
#
run_bash() {
path_to_bash="$1"
if [ -f ${path_to_bash} ]; then
  source ${path_to_bash} 2>/dev/null
else
  echo "File does not exist ${path_to_bash}"
fi
}

run_bash "$MAPR_HOME"/conf/env.sh
run_bash "$MAPR_HOME"/server/common-ecosystem.sh

configure_logging
if [ $? -ne 0 ]; then
  echo "MAPR_HOME seems to not be set correctly or mapr-core not installed"
  exit 1
fi

#
# Initialize environment
#
initCfgEnv

#
# Globals
#
MAPR_LIB="$MAPR_HOME"/lib
SLIDER_VERSION_FILE="$MAPR_HOME"/slider/sliderversion
SLIDER_VERSION=$(cat "$SLIDER_VERSION_FILE")
SLIDER_HOME="$MAPR_HOME"/slider/slider-"$SLIDER_VERSION"
SLIDER_BIN="$SLIDER_HOME"/bin
SLIDER_LIB="$SLIDER_HOME"/lib
SLIDER_CONF="$SLIDER_HOME"/conf
SLIDER_LOGS="$SLIDER_HOME"/logs
SLIDER_PIDS="$SLIDER_HOME"/pids
SLIDER_SERVER="$SLIDER_CONF"/slider-server.xml
SLIDER_CLIENT="$SLIDER_CONF"/slider-client.xml


#
# Find mapr user and group
#
find_mapr_user_and_group() {
if [ -z "$MAPR_USER" ] ; then
  MAPR_USER=$( awk -F = '$1 == "mapr.daemon.user" { print $2 }' "$DAEMON_CONF")
fi
if [ -z "$MAPR_GROUP" ] ; then
  MAPR_GROUP=$( awk -F = '$1 == "mapr.daemon.group" { print $2 }' "$DAEMON_CONF")
fi

if [ -z "$MAPR_USER" ] ; then
  MAPR_USER=mapr
fi
if [ -z "$MAPR_GROUP" ] ; then
  MAPR_GROUP=mapr
fi
logInfo "Mapr User = ['$MAPR_USER'], Mapr Group = ['$MAPR_GROUP']"
}


#
# Grant owners
#
set_admin_user_group_to(){
path=$1
option=$2
if [ "$option" = "recursive" ] ; then
  option="-R"
fi
if [ ! -z "$MAPR_USER" ]; then
  chown ${option} "$MAPR_USER" "$path"
fi
if [ ! -z "$MAPR_GROUP" ]; then
  chgrp ${option} "$MAPR_GROUP" "$path"
fi
logInfo "Set admin user group to ['$path']"
}


#
# Set owner and group for all dirs in $SLIDER_HOME. Skips the contents of dirs in exception list
#
set_all_dirs_owner_group(){
# list of exception where not to change owner/group
for dir in "$SLIDER_HOME"/* ; do
  set_admin_user_group_to "$dir" recursive
done
}

#
# Sets default 0644 permissions to slider-server.xml
#
grant_permissions_to_slider_server(){
chmod 0644 "$SLIDER_SERVER"
logInfo "Set default 0644 permissions to slider.xml"
}


#
# Sets default 0644 permissions to slider-client.xml
#
grant_permissions_to_slider_client(){
chmod 0644 "$SLIDER_CLIENT"
logInfo "Set default 0644 permissions to slider-client.xml"
}



#
# Configures owner/group and read/write permissions of Hive components
#
configure_permissions(){
logInfo "Configuring owner/group and read/write permissions of Hive components ..."
set_all_dirs_owner_group
set_admin_user_group_to "$SLIDER_VERSION_FILE"
set_admin_user_group_to "$SLIDER_HOME"
grant_permissions_to_slider_server
grant_permissions_to_slider_client
logInfo "Permissions has been configured"
}


#
# Copy mapr-fs jar to Slider lib
#
copy_maprfs_jar(){
MAPR_FS_BASENAME_JAR=$(find "$MAPR_LIB" -maxdepth 1 -regextype posix-extended  -regex '.*maprfs-[0-9].*[^tests]\.jar' -exec basename {} +)
if [ -z "$MAPR_FS_BASENAME_JAR" ]; then
  echo "Can't find $MAPR_HOME/lib/maprfs*jar"
  exit 1
fi
MAPR_FS_TARGET_JAR="$SLIDER_LIB/$MAPR_FS_BASENAME_JAR"
MAPR_FS_SOURCE_JAR="$MAPR_LIB/$MAPR_FS_BASENAME_JAR"
if [ ! -f "$MAPR_FS_TARGET_JAR" ]; then
  cp "$MAPR_FS_SOURCE_JAR" "$MAPR_FS_TARGET_JAR"
fi
}

#
# main
#
# typically called from core configure.sh
#

USAGE="Usage: $0 [options]
where options include:
    --secure|-s           configure MapR-SASL security cluster
    --unsecure|-u         configure MapR unsecure cluster
    --customSecure|-c     keep existing security configuration
    --help|-h             print help
    -EC <commonEcoOPts>   clusterwide options like TL, RM, ES
    -R                    Roles only"

if [ $# -gt 0 ]; then
  OPTS=$(getopt -a -o chsuC:R -l EC: -l help -l customSecure -l secure -l unsecure -l R -- "$@")
  if [ $? != 0 ] ; then
      echo -e "${USAGE}"
      return 2 2>/dev/null || exit 2
  fi
  eval set -- "$OPTS"

  while (( "$#" )); do
    case "$1" in
      --EC|-C)
        #Parse Common options
        #Ingore ones we don't care about
        ecOpts=("$2")
        shift 2
        restOpts="$@"
        eval set -- "${ecOpts[*]} --"
        while (( "$#" )); do
          case "$1" in
            --R|-R)
              SLIDER_CONF_ASSUME_RUNNING_CORE=1
              shift 1;;
            --RM|-RM)
              rm_ip=$2;
              shift 2;;
            --TL|-TL)
              tl_ip=$2;
              shift 2;;
            --) shift
              break;;
              *)
              #echo "Ignoring common option $j"
              shift 1;;
          esac
        done
        shift 2
        eval set -- "$restOpts"
        ;;
      --R|-R)
        SLIDER_CONF_ASSUME_RUNNING_CORE=1
        shift 1
        ;;
      --secure|-s)
          shift 1;;
      --customSecure|-c)
          shift 1;;
      --unsecure|-u)
          shift 1;;
      --help|-h)
          echo -e "${USAGE}"
          return 2 2>/dev/null || exit 2
          ;;
      --)
          shift 1;;
        *)
          echo "Unknown option $1"
          echo -e "${USAGE}"
          return 2 2>/dev/null || exit 2
          ;;
    esac
  done
fi

copy_maprfs_jar

find_mapr_user_and_group

configure_permissions
